"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RegistryClient = void 0;
const lodash_1 = __importDefault(require("lodash"));
const client_sts_1 = require("@aws-sdk/client-sts");
const docker_reference_1 = require("@codefresh-io/docker-reference");
const nodegistry_1 = require("nodegistry");
const types_1 = require("./types");
const fs_1 = __importStar(require("fs"));
const { GcrRegistry, EcrRegistry, DockerhubRegistry, StandardRegistry } = nodegistry_1.registries;
class RegistryClient {
    constructor(inputs) {
        this.decodeBase64 = (str) => Buffer.from(str, 'base64').toString();
        this.inputs = inputs;
    }
    checkNotEmpty(testVar) {
        return (testVar && testVar !== types_1.CF_NOT_EXIST);
    }
    parseImageName(imageName) {
        return (0, docker_reference_1.parseFamiliarName)(imageName, docker_reference_1.parseQualifiedNameOptimized);
    }
    getCredentialsFromDockerConfig(image) {
        const dockerConfig = JSON.parse(fs_1.default.readFileSync(this.inputs.dockerConfigPath).toString());
        const imageData = this.parseImageName(image);
        const auths = lodash_1.default.get(dockerConfig, 'auths', {});
        const domainKey = lodash_1.default.findKey(auths, (auth, domain) => {
            if (domain.includes(imageData.domain)) {
                return true;
            }
        });
        if (!domainKey) {
            throw new Error('Domain is not provided.');
        }
        const authInfo = auths[domainKey];
        const auth = this.decodeBase64(authInfo.auth);
        const [username, password] = auth.split(':');
        if (domainKey.includes('docker.io')) {
            return new DockerhubRegistry({
                username,
                password,
            });
        }
        else if (domainKey.includes('azurecr.io')) {
            throw new Error('Azure Container Registry using docker config json is not supported.');
        }
        else if (domainKey.includes('gcr.io')) {
            throw new Error('Google container registry using docker config json is not supported.');
        }
        return new StandardRegistry({
            request: {
                host: domainKey
            },
            credentials: {
                username,
                password,
            },
            ignoreRedirects: true
        });
    }
    async createECRUsingSTS(role, region) {
        console.log(`Retrieving credentials for ECR ${region} using STS token`);
        try {
            const sts = new client_sts_1.STS({ region });
            sts.middlewareStack.add((next, context) => (args) => {
                console.log('\n -- printed from inside STS middleware -- ');
                console.log('hostname:', args.request.hostname);
                console.log('input:', args.input, '\n');
                return next(args);
            }, {
                step: 'build',
            });
            const timestamp = Date.now();
            const params = {
                RoleArn: role,
                RoleSessionName: `be-descriptibe-here-${timestamp}`
            };
            const data = await sts.assumeRole(params);
            return new EcrRegistry({
                promise: Promise,
                credentials: {
                    accessKeyId: data.Credentials?.AccessKeyId,
                    secretAccessKey: data.Credentials?.SecretAccessKey,
                    sessionToken: data.Credentials?.SessionToken,
                    region: region,
                },
            });
        }
        catch (error) {
            console.error(error.message);
            throw error;
        }
    }
    async createRegistryClientByImage(image) {
        const imageData = this.parseImageName(image);
        if (imageData.domain.includes('docker.io')) {
            if (this.checkNotEmpty(this.inputs.dockerhub?.username)
                && this.checkNotEmpty(this.inputs.dockerhub?.password)) {
                return new DockerhubRegistry(this.inputs.dockerhub);
            }
            throw new Error('Registry credentials for DOCKER not set. Add following registry parameters in your workflow to continue:\n - DOCKERHUB_USERNAME\n - DOCKERHUB_PASSWORD\n');
        }
        else if (imageData.domain.includes('gcr.io')) {
            if (this.inputs.gcr?.keyFilePath) {
                return new GcrRegistry({
                    keyfile: fs_1.default.readFileSync(this.inputs.gcr.keyFilePath),
                    request: { host: 'gcr.io' }
                });
            }
            throw new Error('Registry credentials for GCR not set. Add following registry parameters in your workflow to continue:\n - GCR_KEY_FILE_PATH\n');
        }
        else if (imageData.domain.includes('azurecr.io')) {
            throw new Error('Azure Container Registry using docker config json is not supported.');
        }
        else if (imageData.domain.includes('ecr')) {
            if (this.checkNotEmpty(this.inputs.aws?.role)
                && this.checkNotEmpty(this.inputs.aws?.credentials?.region)) {
                return this.createECRUsingSTS(this.inputs.aws?.role, this.inputs.aws?.credentials?.region);
            }
            else if (this.checkNotEmpty(this.inputs.aws?.credentials?.accessKeyId)
                && this.checkNotEmpty(this.inputs.aws?.credentials?.secretAccessKey)
                && this.checkNotEmpty(this.inputs.aws?.credentials?.region)) {
                return new EcrRegistry({
                    promise: Promise,
                    credentials: this.inputs.aws?.credentials,
                });
            }
            throw new Error('Registry credentials for ECR not set. Add following registry parameters in your workflow to continue:\n - AWS_ACCESS_KEY\n - AWS_SECRET_KEY\n - AWS_REGION\n');
        }
        if (this.checkNotEmpty(this.inputs.generic?.credentials?.username)
            && this.checkNotEmpty(this.inputs.generic?.credentials?.password)
            && this.checkNotEmpty(this.inputs.generic?.request?.host)) {
            lodash_1.default.set(this.inputs, 'generic.ignoreRedirects', true);
            return new StandardRegistry(this.inputs.generic);
        }
        if (this.inputs.dockerConfigPath) {
            return this.getCredentialsFromDockerConfig(image);
        }
        throw new Error('Registry credentials is required parameter. Add one from following registry parameters in your workflow to continue:\n - Docker credentials: DOCKERHUB_USERNAME, DOCKERHUB_PASSWORD\n - GCR credentials: GCR_KEY_FILE_PATH\n - AWS registry credentials: AWS_ACCESS_KEY, AWS_SECRET_KEY, AWS_REGION\n - Standard registry credentials: REGISTRY_USERNAME, REGISTRY_PASSWORD, REGISTRY_DOMAIN\n');
    }
    async createRegistryClient(image) {
        if (this.inputs.dockerConfigPath) {
            return this.getCredentialsFromDockerConfig(image);
        }
        if (this.checkNotEmpty(this.inputs.dockerhub?.username)
            && this.checkNotEmpty(this.inputs.dockerhub?.password)) {
            return new DockerhubRegistry(this.inputs.dockerhub);
        }
        if (this.checkNotEmpty(this.inputs.generic?.credentials?.username)
            && this.checkNotEmpty(this.inputs.generic?.credentials?.password)
            && this.checkNotEmpty(this.inputs.generic?.request?.host)) {
            lodash_1.default.set(this.inputs, 'generic.ignoreRedirects', true);
            return new StandardRegistry(this.inputs.generic);
        }
        if (this.checkNotEmpty(this.inputs.aws?.role)
            && this.checkNotEmpty(this.inputs.aws?.credentials?.region)) {
            return this.createECRUsingSTS(this.inputs.aws?.role, this.inputs.aws?.credentials?.region);
        }
        if (this.inputs.gcr?.keyFilePath) {
            return new GcrRegistry({
                keyfile: fs_1.default.readFileSync(this.inputs.gcr.keyFilePath),
                request: { host: 'gcr.io' }
            });
        }
        if (this.checkNotEmpty(this.inputs.aws?.credentials?.accessKeyId)
            && this.checkNotEmpty(this.inputs.aws?.credentials?.secretAccessKey)
            && this.checkNotEmpty(this.inputs.aws?.credentials?.region)) {
            return new EcrRegistry({
                promise: Promise,
                credentials: this.inputs.aws?.credentials,
            });
        }
        throw new Error('Registry credentials is required parameter. Add one from following registry parameters in your workflow to continue:\n - Docker credentials: DOCKERHUB_USERNAME, DOCKERHUB_PASSWORD\n - GCR credentials: GCR_KEY_FILE_PATH\n - AWS registry credentials: AWS_ACCESS_KEY, AWS_SECRET_KEY, AWS_REGION\n - Standard registry credentials: REGISTRY_USERNAME, REGISTRY_PASSWORD, REGISTRY_DOMAIN');
    }
    async getRegistryClient(image) {
        if (this.inputs.retrieveCredentialsByDomain) {
            return this.createRegistryClientByImage(image);
        }
        return this.createRegistryClient(image);
    }
    getLayersFromHistory(history, layers) {
        let shell = '/bin/sh -c';
        const instructionPrefix = '#(nop)';
        const layersSizes = lodash_1.default.map(layers, l => l.size);
        history = history.reverse().map((row) => {
            row = row || {};
            const CreatedBy = lodash_1.default.get(row, 'created_by', '');
            let instruction, args;
            if (!CreatedBy.includes(instructionPrefix)) {
                instruction = "RUN";
                args = CreatedBy.replace(shell, '').trim();
            }
            else {
                const cmd = CreatedBy.slice(CreatedBy.indexOf(instructionPrefix) + instructionPrefix.length).trim();
                const nextSpacePosition = cmd.indexOf(' ');
                instruction = cmd.slice(0, Math.max(0, nextSpacePosition)).trim();
                args = cmd.slice(nextSpacePosition).trim();
                if (instruction === "SHELL") {
                    shell = (args[0] === '[' && args[args.length - 1] === ']') ? (args
                        .slice(1, -1)
                        .replace(/["']/g, '')
                        .split(',')
                        .map(part => { return part.trim(); })
                        .join(' ')) : args;
                }
            }
            return {
                created: row.created,
                size: row.empty_layer ? 0 : layersSizes.pop(),
                instruction,
                args
            };
        }).reverse();
        return history;
    }
    async buildDockerfile() {
        try {
            const { dockerfileContent, dockerfilePath } = this.inputs;
            if (dockerfileContent) {
                return Buffer.from(dockerfileContent, 'base64').toString();
            }
            const file = await fs_1.promises.readFile(dockerfilePath ? dockerfilePath : '/tmp/Dockerfile');
            return file.toString();
        }
        catch (error) {
            console.error(`Can't get Dockerfile. ${error.message}`);
            return '';
        }
    }
    getRegistryImageSize(manifest) {
        return manifest.config.size + lodash_1.default.reduce(manifest.layers, (sum, layer) => {
            return sum + layer.size;
        }, 0);
    }
}
exports.RegistryClient = RegistryClient;
//# sourceMappingURL=registries.js.map